@@ -1,5 +1,5 @@
-import numpy as np  # type: ignore
-from typing import List, Optional
+from numpy.typing import NDArray
+from typing import List
 
 from uniplot.discretizer import discretize
 
@@ -13,15 +13,19 @@ class LabelSet:
 
     def __init__(
         self,
-        labels: np.array,
+        labels: NDArray,
         x_min: float,
         x_max: float,
         available_space: int,
+        unit: str = "",
+        log: bool = False,
         vertical_direction: bool = False,
     ):
         self.labels = labels
         self.x_min = x_min
         self.x_max = x_max
+        self.unit = unit
+        self.log = log
         self.available_space = available_space
         self.vertical_direction = vertical_direction
         self._results_already_in_cache: bool = False
@@ -52,7 +56,7 @@ class LabelSet:
             lines: List[str] = [""] * self.available_space
 
             for i, label in enumerate(self.labels):
-                str_label = str_labels[i]
+                str_label = self._add_log_to_label(str_labels[i]) + self.unit
                 index = (
                     self.available_space
                     - 1
@@ -80,7 +84,7 @@ class LabelSet:
             # So this is for the x axis case
             line = ""
             for i, label in enumerate(self.labels):
-                str_label = str_labels[i]
+                str_label = self._add_log_to_label(str_labels[i]) + self.unit
                 offset = max(
                     0,
                     discretize(
@@ -102,6 +106,7 @@ class LabelSet:
                     buffer = 1
                     self._render_does_overlap = True
 
+                # Compose string for this line
                 line = line + (" " * buffer) + str_label
 
             self._rendered_result = [line]
@@ -109,7 +114,7 @@ class LabelSet:
 
     def _find_shortest_string_representation(
         self,
-        numbers: List[Optional[float]],
+        numbers: NDArray,
     ) -> List[str]:
         """
         This method will find the shortest numerical values for axis labels that are different from each other.
@@ -128,7 +133,7 @@ class LabelSet:
         # Fallback to naive string conversion
         return ["" if n is None else str(n) for n in numbers]
 
-    def _float_format(self, n: float, nr_digits: int):
+    def _float_format(self, n: float, nr_digits: int) -> str:
         """
         Format a number to a specified precision.
 
@@ -136,4 +141,19 @@ class LabelSet:
         """
         if nr_digits == 0:
             return ("{:,d}").format(int(n))
-        return ("{:,." + str(nr_digits) + "g}").format(float(n))
+        return ("{:,." + str(nr_digits) + "f}").format(float(n))
+
+    def _add_log_to_label(self, label) -> str:
+        if not self.log:
+            return label
+
+        # What follows is a bit of a hack
+        if label == "0":
+            return "1"
+        if label == "1":
+            return "10"
+        if label == "2":
+            return "100"
+        if label == "-1":
+            return "0.1"
+        return "10^" + label
