@@ -1,5 +1,5 @@
 import numpy as np  # type: ignore
-from typing import Dict, Tuple, List, Optional, Any
+from typing import Dict
 
 from uniplot.multi_series import MultiSeries
 from uniplot.options import Options
@@ -15,6 +15,19 @@ def validate_and_transform_options(series: MultiSeries, kwargs: Dict = {}) -> Op
 
     As a result the somewhat hacky code below should at least be confined to this function, and not spread throughout uniplot.
     """
+    if kwargs.get("x_as_log"):
+        series.xs = [np.log10(x) for x in series.xs]
+        if not kwargs.get("x_gridlines"):
+            kwargs["x_gridlines"] = []
+        else:
+            kwargs["x_gridlines"] = list(np.log10(np.array(kwargs["x_gridlines"])))
+    if kwargs.get("y_as_log"):
+        series.ys = [np.log10(y) for y in series.ys]
+        if not kwargs.get("y_gridlines"):
+            kwargs["y_gridlines"] = []
+        else:
+            kwargs["y_gridlines"] = list(np.log10(np.array(kwargs["y_gridlines"])))
+
     # Set x bounds to show all points by default
     x_enlarge_delta = AUTO_WINDOW_ENLARGE_FACTOR * (series.x_max() - series.x_min())
     kwargs["x_min"] = kwargs.get("x_min", series.x_min() - x_enlarge_delta)
