@@ -1,12 +1,16 @@
-import numpy as np  # type: ignore
+import numpy as np
+from numpy.typing import NDArray
 from typing import Optional
 
-from uniplot.discretizer import discretize, invert_discretize
+from uniplot.discretizer import (
+    discretize_array,
+    invert_discretize_array,
+)
 
 
 def render(
-    xs: np.array,
-    ys: np.array,
+    xs: NDArray,
+    ys: NDArray,
     x_min: float,
     x_max: float,
     y_min: float,
@@ -14,7 +18,7 @@ def render(
     width: int,
     height: int,
     lines: bool = False,
-) -> np.array:
+) -> NDArray:
     """
     Turn a list of 2D points into a raster matrix.
 
@@ -34,8 +38,8 @@ def render(
 
     pixels = np.zeros((height, width), dtype=int)
 
-    x_indices = discretize(xs, x_min, x_max, steps=width)
-    y_indices = discretize(ys, y_min, y_max, steps=height)
+    x_indices = discretize_array(xs, x_min, x_max, steps=width)
+    y_indices = discretize_array(ys, y_min, y_max, steps=height)
 
     # Invert y direction to optimize for plotting later
     y_indices = (height - 1) - y_indices
@@ -57,6 +61,11 @@ def render(
             (xy_indices[:-1], xys[:-1], xy_indices[1:], xys[1:]), axis=1
         ).astype(float)
 
+        # Filter any line segments where any coordinatwe is NaN
+        xy_line_endpoints = xy_line_endpoints[
+            ~np.isnan(xy_line_endpoints).any(axis=2).any(axis=1)
+        ]
+
         # Filter out of view line segments
         xy_line_endpoints = xy_line_endpoints[
             (
@@ -92,7 +101,7 @@ def render(
 
         # TODO This can likely be optimized by assembling all segments and computing the
         # pixels of all lines together, or at least of each half split by slope
-        # for segment in np.nditer(xy_line_endpoints): # TODO use this
+        # for segment in np.nditer(xy_line_endpoints)
         for segment in xy_line_endpoints:
             [
                 [x_index_start, y_index_start],
@@ -107,6 +116,12 @@ def render(
             y_index_start = int(round(y_index_start))
             y_index_stop = int(round(y_index_stop))
 
+            # For convenience
+            x_index_smaller = min(x_index_start, x_index_stop)
+            x_index_bigger = max(x_index_start, x_index_stop)
+            y_index_smaller = min(y_index_start, y_index_stop)
+            y_index_bigger = max(y_index_start, y_index_stop)
+
             # Slope is inverted because y indices are inverted
             indices_slope: Optional[float] = None
             if x_index_start != x_index_stop:
@@ -123,19 +138,9 @@ def render(
                 step = 1
                 if y_index_stop < y_index_start:
                     step = -1
-                # DEBUG
-                try:
-                    pixels[y_index_start:y_index_stop:step, x_index_start] = 1
-                except:
-                    print(
-                        "PROBLEM: values:",
-                        y_index_start,
-                        y_index_stop,
-                        step,
-                        x_index_start,
-                    )
-                    raise
+                pixels[y_index_start:y_index_stop:step, x_index_start] = 1
                 pixels_already_drawn = True
+
             elif y_index_start == y_index_stop:
                 # That means it's a horizontal line
                 step = 1
@@ -143,6 +148,7 @@ def render(
                     step = -1
                 pixels[y_index_start, x_index_start:x_index_stop:step] = 1
                 pixels_already_drawn = True
+
             elif abs(indices_slope) > 1:
                 # Draw line by iterating vertically
                 # 1. Compute y indices in the middle of bins between the two origins
@@ -150,9 +156,9 @@ def render(
                 if y_index_stop < y_index_start:
                     step = -1
                 y_indices_of_line = np.arange(
-                    y_index_start + step, y_index_stop, step=step
+                    y_index_start, y_index_stop + step, step=step
                 )
-                ys_of_line = invert_discretize(
+                ys_of_line = invert_discretize_array(
                     height - 1 - y_indices_of_line,
                     minimum=y_min,
                     maximum=y_max,
@@ -164,14 +170,10 @@ def render(
                 #   xs_of_line - x_start = (ys_of_line - y_start) / slope
                 #   xs_of_line = (ys_of_line - y_start) / slope + x_start
                 xs_of_line = (ys_of_line - y_start) / slope + x_start
-                x_indices_of_line = discretize(
+                x_indices_of_line = discretize_array(
                     xs_of_line, x_min=x_min, x_max=x_max, steps=width
                 )
 
-                # 3. Draw pixels
-                xy_indices_of_line = np.column_stack(
-                    (x_indices_of_line, y_indices_of_line)
-                )
             else:
                 # Draw line by iterating horizontically
                 # 1. Compute x indices in the middle of bins between the two origins
@@ -179,9 +181,9 @@ def render(
                 if x_index_stop < x_index_start:
                     step = -1
                 x_indices_of_line = np.arange(
-                    x_index_start + step, x_index_stop, step=step
+                    x_index_start, x_index_stop + step, step=step
                 )
-                xs_of_line = invert_discretize(
+                xs_of_line = invert_discretize_array(
                     x_indices_of_line, minimum=x_min, maximum=x_max, nr_bins=width
                 )
 
@@ -190,27 +192,29 @@ def render(
                 y_indices_of_line = (
                     height
                     - 1
-                    - discretize(ys_of_line, x_min=y_min, x_max=y_max, steps=height)
+                    - discretize_array(
+                        ys_of_line, x_min=y_min, x_max=y_max, steps=height
+                    )
                 )
 
-            # Finally, draw pixels (of needed)
+            # Finally, draw pixels (if needed)
             if not pixels_already_drawn:
+                # Assemble pixels
                 xy_indices_of_line = np.column_stack(
                     (x_indices_of_line, y_indices_of_line)
                 )
 
                 # Filter out of view pixels
-                # TODO DRY
                 xy_indices_of_line = xy_indices_of_line[
-                    (xy_indices_of_line[:, 0] >= 0)
-                    & (xy_indices_of_line[:, 0] < width)
-                    & (xy_indices_of_line[:, 1] >= 0)
-                    & (xy_indices_of_line[:, 1] < height)
+                    (xy_indices_of_line[:, 0] >= max(0, x_index_smaller))
+                    & (xy_indices_of_line[:, 0] <= min(width - 1, x_index_bigger))
+                    & (xy_indices_of_line[:, 1] >= max(0, y_index_smaller))
+                    & (xy_indices_of_line[:, 1] <= min(height - 1, y_index_bigger))
                 ]
                 xy_indices_of_line = xy_indices_of_line.T
                 pixels[xy_indices_of_line[1], xy_indices_of_line[0]] = 1
 
-    # Filter out of view pixels
+    # Filter out NaN and out of view pixels
     xy_indices = xy_indices[
         (xy_indices[:, 0] >= 0)
         & (xy_indices[:, 0] < width)
@@ -226,12 +230,12 @@ def render(
 
 
 def merge_on_top(
-    low_layer: np.array,
-    high_layer: np.array,
+    low_layer: NDArray,
+    high_layer: NDArray,
     width: int,
     height: int,
     with_shadow: bool = False,
-) -> np.array:
+) -> NDArray:
     """
     Put a pixel matrix on top of another, with an optional single solid line of "shadow",
     including diagonal fields.
