@@ -1,4 +1,5 @@
-import numpy as np  # type: ignore
+import numpy as np
+from numpy.typing import NDArray
 from typing import Tuple, List, Optional, Any
 
 from uniplot.multi_series import MultiSeries
@@ -59,43 +60,18 @@ def plot(ys: Any, xs: Optional[Any] = None, **kwargs) -> None:
             print("Move h/j/k/l, zoom u/n, or r to reset. ESC/q to quit")
             key_pressed = getch().lower()
 
-            # TODO Move all of the below to the `Options` class
             if key_pressed == "h":
-                # Left
-                step = 0.1 * (options.x_max - options.x_min)
-                options.x_min = options.x_min - step
-                options.x_max = options.x_max - step
+                options.shift_view_left()
             elif key_pressed == "l":
-                # Right
-                step = 0.1 * (options.x_max - options.x_min)
-                options.x_min = options.x_min + step
-                options.x_max = options.x_max + step
+                options.shift_view_right()
             elif key_pressed == "j":
-                # Up
-                step = 0.1 * (options.y_max - options.y_min)
-                options.y_min = options.y_min - step
-                options.y_max = options.y_max - step
+                options.shift_view_down()
             elif key_pressed == "k":
-                # Down
-                step = 0.1 * (options.y_max - options.y_min)
-                options.y_min = options.y_min + step
-                options.y_max = options.y_max + step
+                options.shift_view_up()
             elif key_pressed == "u":
-                # Zoom in
-                step = 0.1 * (options.x_max - options.x_min)
-                options.x_min = options.x_min + step
-                options.x_max = options.x_max - step
-                step = 0.1 * (options.y_max - options.y_min)
-                options.y_min = options.y_min + step
-                options.y_max = options.y_max - step
+                options.zoom_in()
             elif key_pressed == "n":
-                # Zoom out
-                step = 0.1 * (options.x_max - options.x_min)
-                options.x_min = options.x_min - step
-                options.x_max = options.x_max + step
-                step = 0.1 * (options.y_max - options.y_min)
-                options.y_min = options.y_min - step
-                options.y_max = options.y_max + step
+                options.zoom_out()
             elif key_pressed == "r":
                 options.reset_view()
             elif key_pressed in ["q", "\x1b"]:
@@ -162,7 +138,7 @@ def histogram(
     xs_histo_series = []
     ys_histo_series = []
     for s in multi_series.ys:
-        hist, bin_edges = np.histogram(s, bins=bins, range=(bins_min, bins_max))
+        hist, bin_edges = np.histogram(s, bins=bins)
 
         # Draw vertical and horizontal lines to connect points
         xs_here = np.zeros(1 + 2 * bins + 1)
@@ -197,7 +173,7 @@ def _generate_header(options: Options) -> List[str]:
 def _generate_body(
     x_axis_labels: str,
     y_axis_labels: List[str],
-    pixel_character_matrix: np.array,
+    pixel_character_matrix: NDArray,
     options: Options,
 ) -> List[str]:
     """
@@ -222,7 +198,7 @@ def _generate_body(
 
 def _generate_body_raw_elements(
     series: MultiSeries, options: Options
-) -> Tuple[str, List[str], np.array]:
+) -> Tuple[str, List[str], NDArray]:
     """
     Generates the x-axis labels, y-axis labels, and the pixel character matrix.
     """
@@ -231,24 +207,40 @@ def _generate_body_raw_elements(
         x_min=options.y_min,
         x_max=options.y_max,
         available_space=options.height,
+        unit=options.y_unit,
+        log=options.y_as_log,
         vertical_direction=True,
     )
-    y_axis_labels = ["-error generating labels, sorry-"] * options.height
+    y_axis_labels = [""] * options.height
     if y_axis_label_set is not None:
         y_axis_labels = y_axis_label_set.render()
 
+    # Observe line_length_hard_cap
+    if options.line_length_hard_cap is not None:
+        options.reset_width()
+        # Determine maximum length of y axis label
+        max_y_label_length = max([len(l) for l in y_axis_labels])
+        # Make sure the total plot does not exceed `line_length_hard_cap`
+        if 2 + options.width + 1 + max_y_label_length > options.line_length_hard_cap:
+            # Overflow, so we need to reduce width of plot area
+            options.width = options.line_length_hard_cap - (2 + 1 + max_y_label_length)
+            if options.width < 1:
+                raise
+
     # Prepare x axis labels
     x_axis_label_set = extended_talbot_labels(
         x_min=options.x_min,
         x_max=options.x_max,
         available_space=options.width,
+        unit=options.x_unit,
+        log=options.x_as_log,
         vertical_direction=False,
     )
-    x_axis_labels = "-error generating labels, sorry-"
+    x_axis_labels = ""
     if x_axis_label_set is not None:
         x_axis_labels = x_axis_label_set.render()[0]
 
-    # Prefare graph surface
+    # Prepare graph surface
     pixel_character_matrix = layer_assembly.assemble_scatter_plot(
         xs=series.xs, ys=series.ys, options=options
     )
